@@ -6,7 +6,6 @@ setMethod("recode","item",function(x,...,
                                    otherwise=NA){
   recodings <- match.call(expand.dots=FALSE)$...
   recodings <- recodings[nzchar(sapply(recodings,paste,collapse=""))]
-
   if(length(otherwise)!=1) stop("otherwise= argument must have length 1")
   
   if(any(sapply(sapply(recodings,"[[",1),paste)!="<-"))
@@ -36,17 +35,18 @@ setMethod("recode","item",function(x,...,
   }
   else
       oldcodes <- sapply(conditions,eval)
+  NA_recoded <- any(unlist(suppressWarnings(sapply(oldcodes,is.na))))
       
   conditions[has.range] <- lapply(conditions[has.range],
                           function(x)
                           as.call(list(as.symbol("&"),
-                            as.call(list(as.symbol("<="),x[[2]],as.symbol("x"))),
-                            as.call(list(as.symbol("<="),as.symbol("x"),x[[3]]))
+                            as.call(list(as.symbol("<="),x[[2]],quote(x@.Data))),
+                            as.call(list(as.symbol("<="),quote(x@.Data),x[[3]]))
                             )
                           ))
   conditions[!has.range] <- lapply(conditions[!has.range],
                           function(x)
-                          as.call(list(as.symbol("%in%"),as.symbol("x"),x))
+                          as.call(list(as.symbol("%in%"),quote(x@.Data),x))
                           )
   torecode <- sapply(conditions,eval,envir=environment(),enclos=parent.frame())
   if(!is.matrix(torecode)) torecode <- t(torecode)
@@ -86,7 +86,9 @@ setMethod("recode","item",function(x,...,
     nNA <-sum(is.na(otherwise[!recoded])) - sum(is.na(x[!recoded]))
     if(nNA > 0)
       warning("recoding created ",nNA," NAs")
-  }
+    if(!NA_recoded)
+      y[is.na(x)] <- NA
+}
   newvlab <- newcodes[nzchar(names(newcodes))]
   bijective <- FALSE
   lab.y <- lab.x <- labels(x)
@@ -162,6 +164,7 @@ setMethod("recode","vector",function(x,...,
     if(!length(recodings)) return(x)
     newcodes <- lapply(recodings,"[[",2)
     oldcodes <- lapply(recodings,"[[",3)
+    NA_recoded <- any(unlist(suppressWarnings(sapply(oldcodes,is.na))))
     has.range <- paste(lapply(oldcodes,"[[",1)) == "range"
     if(any(has.range)){
         has.min <- paste(lapply(oldcodes[has.range],"[[",2)) == "min"
@@ -223,6 +226,8 @@ setMethod("recode","vector",function(x,...,
         nNA <- sum(is.na(otherwise[!recoded])) - sum(is.na(x[!recoded]))
         if(nNA > 0)
             warning("recoding created ",nNA," NAs")
+        if(!NA_recoded)
+            y[is.na(x)] <- NA
     }
     if(is.character(y)) {
         levels <- sort(unique(y[y%nin%newcodes]))
